@@ -361,7 +361,7 @@ class Tortoise:
         discovered_models = []
         possible_models = getattr(module, "__models__", None)
         try:
-            possible_models = [*possible_models]
+            possible_models = [*possible_models]  # type:ignore
         except TypeError:
             possible_models = None
         if not possible_models:
@@ -709,4 +709,4 @@ def run_async(coro: Coroutine) -> None:
         loop.run_until_complete(Tortoise.close_connections())
 
 
-__version__ = "0.17.7"
+__version__ = "0.18.0"
