@@ -23,7 +23,7 @@ from pypika import JoinType, Parameter, Query, Table
 from pypika.terms import ArithmeticExpression, Function
 
 from tortoise.exceptions import OperationalError
-from tortoise.expressions import F
+from tortoise.expressions import F, RawSQL
 from tortoise.fields.base import Field
 from tortoise.fields.relational import (
     BackwardFKRelation,
@@ -32,6 +32,7 @@ from tortoise.fields.relational import (
     RelationalField,
 )
 from tortoise.query_utils import QueryModifier
+from tortoise.utils import chunk
 
 if TYPE_CHECKING:  # pragma: nocoverage
     from tortoise.backends.base.client import BaseDBAsyncClient
@@ -122,7 +123,9 @@ class BaseExecutor:
         sql = " ".join((self.EXPLAIN_PREFIX, query.get_sql()))
         return (await self.db.execute_query(sql))[1]
 
-    async def execute_select(self, query: Query, custom_fields: Optional[list] = None) -> list:
+    async def execute_select(
+        self, query: Union[Query, RawSQL], custom_fields: Optional[list] = None
+    ) -> list:
         _, raw_results = await self.db.execute_query(query.get_sql())
         instance_list = []
         for row in raw_results:
@@ -134,7 +137,7 @@ class BaseExecutor:
                 instance: "Model" = self.model._init_from_db(
                     **dict(zip(keys[:current_idx], values[:current_idx]))
                 )
-                instances: dict[Any, Any] = {path: instance}
+                instances: Dict[Any, Any] = {path: instance}
                 for (
                     model,
                     index,
@@ -192,15 +195,20 @@ class BaseExecutor:
             return cls.TO_DB_OVERRIDE[field_object.__class__](field_object, attr, instance)
         return field_object.to_db_value(attr, instance)
 
-    def _prepare_insert_statement(self, columns: Sequence[str], has_generated: bool = True) -> str:
+    def _prepare_insert_statement(
+        self, columns: Sequence[str], has_generated: bool = True, ignore_conflicts: bool = False
+    ) -> str:
         # Insert should implement returning new id to saved object
         # Each db has it's own methods for it, so each implementation should
         # go to descendant executors
-        return str(
+        query = (
             self.db.query_class.into(self.model._meta.basetable)
             .columns(*columns)
             .insert(*[self.parameter(i) for i in range(len(columns))])
         )
+        if ignore_conflicts:
+            query = query.ignore()
+        return str(query)
 
     async def _process_insert_result(self, instance: "Model", results: Any) -> None:
         raise NotImplementedError()  # pragma: nocoverage
@@ -224,21 +232,12 @@ class BaseExecutor:
             ]
             await self.db.execute_insert(self.insert_query_all, values)
 
-    @staticmethod
-    def _chunk(instances: "Iterable[Model]", batch_size: Optional[int] = None):
-        if not batch_size:
-            yield instances
-        else:
-            instances = list(instances)
-            for i in range(0, len(instances), batch_size):
-                yield instances[i : i + batch_size]  # noqa:E203
-
     async def execute_bulk_insert(
         self,
         instances: "Iterable[Model]",
         batch_size: Optional[int] = None,
     ) -> None:
-        for instance_chunk in self._chunk(instances, batch_size):
+        for instance_chunk in chunk(instances, batch_size):
             values_lists_all = []
             values_lists = []
             for instance in instance_chunk:
@@ -454,10 +453,10 @@ class BaseExecutor:
             joined_tables: List[Table] = []
             modifier = QueryModifier()
             for node in related_query._q_objects:
+                node._annotations = related_query._annotations
+                node._custom_filters = related_query._custom_filters
                 modifier &= node.resolve(
                     model=related_query.model,
-                    annotations=related_query._annotations,
-                    custom_filters=related_query._custom_filters,
                     table=related_query_table,
                 )
 
