@@ -1,4 +1,3 @@
-import logging
 from hashlib import sha256
 from typing import TYPE_CHECKING, Any, List, Set, Type, cast
 
@@ -13,8 +12,6 @@ if TYPE_CHECKING:  # pragma: nocoverage
 
 # pylint: disable=R0201
 
-logger = logging.getLogger("tortoise")
-
 
 class BaseSchemaGenerator:
     DIALECT = "sql"
