@@ -48,7 +48,15 @@ from tortoise.filters import get_filters_for_field
 from tortoise.functions import Function
 from tortoise.indexes import Index
 from tortoise.manager import Manager
-from tortoise.queryset import ExistsQuery, Q, QuerySet, QuerySetSingle
+from tortoise.queryset import (
+    BulkCreateQuery,
+    BulkUpdateQuery,
+    ExistsQuery,
+    Q,
+    QuerySet,
+    QuerySetSingle,
+    RawSQLQuery,
+)
 from tortoise.router import router
 from tortoise.signals import Signals
 from tortoise.transactions import current_transaction_map, in_transaction
@@ -661,7 +669,7 @@ class Model(metaclass=ModelMeta):
             if callable(field_object.default):
                 setattr(self, key, field_object.default())
             else:
-                setattr(self, key, field_object.default)
+                setattr(self, key, deepcopy(field_object.default))
 
     def _set_kwargs(self, kwargs: dict) -> Set[str]:
         meta = self._meta
@@ -1109,13 +1117,54 @@ class Model(metaclass=ModelMeta):
         await instance.save(using_db=db, force_create=True)
         return instance
 
+    @classmethod
+    def bulk_update(
+        cls: Type[MODEL],
+        objects: Iterable[MODEL],
+        fields: Iterable[str],
+        batch_size: Optional[int] = None,
+    ) -> "BulkUpdateQuery":
+        """
+        Update the given fields in each of the given objects in the database.
+        This method efficiently updates the given fields on the provided model instances, generally with one query.
+
+        .. code-block:: python3
+
+            users = [
+                await User.create(name="...", email="..."),
+                await User.create(name="...", email="...")
+            ]
+            users[0].name = 'name1'
+            users[1].name = 'name2'
+
+            await User.bulk_update(users, fields=['name'])
+
+        :param objects: List of objects to bulk create
+        :param fields: The fields to update
+        :param batch_size: How many objects are created in a single query
+        """
+        return cls._meta.manager.get_queryset().bulk_update(objects, fields, batch_size)
+
+    @classmethod
+    async def in_bulk(
+        cls: Type[MODEL], id_list: Iterable[Union[str, int]], field_name: str = "pk"
+    ) -> Dict[str, MODEL]:
+        """
+        Return a dictionary mapping each of the given IDs to the object with
+        that ID. If `id_list` isn't provided, evaluate the entire QuerySet.
+
+        :param id_list: A list of field values
+        :param field_name: Must be a unique field
+        """
+        return await cls._meta.manager.get_queryset().in_bulk(id_list, field_name)
+
     @classmethod
     async def bulk_create(
         cls: Type[MODEL],
         objects: Iterable[MODEL],
         batch_size: Optional[int] = None,
-        using_db: Optional[BaseDBAsyncClient] = None,
-    ) -> None:
+        ignore_conflicts: bool = False,
+    ) -> "BulkCreateQuery":
         """
         Bulk insert operation:
 
@@ -1126,7 +1175,7 @@ class Model(metaclass=ModelMeta):
 
             e.g. ``IntField`` primary keys will not be populated.
 
-        This is recommend only for throw away inserts where you want to ensure optimal
+        This is recommended only for throw away inserts where you want to ensure optimal
         insert performance.
 
         .. code-block:: python3
@@ -1136,12 +1185,11 @@ class Model(metaclass=ModelMeta):
                 User(name="...", email="...")
             ])
 
+        :param ignore_conflicts: Ignore conflicts when inserting
         :param objects: List of objects to bulk create
         :param batch_size: How many objects are created in a single query
-        :param using_db: Specific DB connection to use instead of default bound
         """
-        db = using_db or cls._choose_db(True)
-        await db.executor_class(model=cls, db=db).execute_bulk_insert(objects, batch_size)
+        return cls._meta.manager.get_queryset().bulk_create(objects, batch_size, ignore_conflicts)
 
     @classmethod
     def first(cls: Type[MODEL]) -> QuerySetSingle[Optional[MODEL]]:
@@ -1203,6 +1251,19 @@ class Model(metaclass=ModelMeta):
         """
         return cls._meta.manager.get_queryset().get(*args, **kwargs)
 
+    @classmethod
+    def raw(cls, sql: str) -> "RawSQLQuery":
+        """
+        Executes a RAW SQL and returns the result
+
+        .. code-block:: python3
+
+            result = await User.raw("select * from users where name like '%test%'")
+
+        :param sql: The raw sql.
+        """
+        return cls._meta.manager.get_queryset().raw(sql)
+
     @classmethod
     def exists(cls: Type[MODEL], *args: Q, **kwargs: Any) -> ExistsQuery:
         """
