@@ -22,7 +22,7 @@ from typing import (
 from pypika import JoinType, Order, Table
 from pypika.functions import Count
 from pypika.queries import QueryBuilder
-from pypika.terms import Term, ValueWrapper
+from pypika.terms import Case, Field, Term, ValueWrapper
 from typing_extensions import Protocol
 
 from tortoise.backends.base.client import BaseDBAsyncClient, Capabilities
@@ -33,15 +33,16 @@ from tortoise.exceptions import (
     MultipleObjectsReturned,
     ParamsError,
 )
-from tortoise.expressions import F
+from tortoise.expressions import Expression, F, Q, RawSQL
 from tortoise.fields.relational import (
     ForeignKeyFieldInstance,
     OneToOneFieldInstance,
     RelationalField,
 )
 from tortoise.functions import Function
-from tortoise.query_utils import Prefetch, Q, QueryModifier, _get_joins_for_related_field
+from tortoise.query_utils import Prefetch, QueryModifier, _get_joins_for_related_field
 from tortoise.router import router
+from tortoise.utils import chunk
 
 # Empty placeholder - Should never be edited.
 
@@ -91,7 +92,7 @@ class AwaitableQuery(Generic[MODEL]):
         self.query: QueryBuilder = QUERY
         self._db: BaseDBAsyncClient = None  # type: ignore
         self.capabilities: Capabilities = model._meta.db.capabilities
-        self._annotations: Dict[str, Function] = {}
+        self._annotations: Dict[str, Expression] = {}
 
     def _choose_db(self, for_write: bool = False) -> BaseDBAsyncClient:
         """
@@ -126,7 +127,9 @@ class AwaitableQuery(Generic[MODEL]):
 
         modifier = QueryModifier()
         for node in q_objects:
-            modifier &= node.resolve(model, annotations, custom_filters, model._meta.basetable)
+            node._annotations = annotations
+            node._custom_filters = custom_filters
+            modifier &= node.resolve(model, model._meta.basetable)
 
         where_criterion, joins, having_criterion = modifier.get_query_modifiers()
         for join in joins:
@@ -319,7 +322,7 @@ class QuerySet(AwaitableQuery[MODEL]):
         self._use_indexes: Set[str] = set()
 
     def _clone(self) -> "QuerySet[MODEL]":
-        queryset = self.__class__.__new__(QuerySet)
+        queryset = self.__class__.__new__(self.__class__)
         queryset.fields = self.fields
         queryset.model = self.model
         queryset.query = self.query
@@ -471,7 +474,7 @@ class QuerySet(AwaitableQuery[MODEL]):
             return queryset
         return self
 
-    def annotate(self, **kwargs: Union[Function, Term]) -> "QuerySet[MODEL]":
+    def annotate(self, **kwargs: Union[Expression, Term]) -> "QuerySet[MODEL]":
         """
         Annotate result with aggregation or function result.
 
@@ -501,7 +504,9 @@ class QuerySet(AwaitableQuery[MODEL]):
         """
         Make QuerySet returns list of tuples for given args instead of objects.
 
-        If ```flat=True`` and only one arg is passed can return flat list.
+        If call after `.get()`, `.get_or_none()` or `.first()` return tuples for given args instead of object.
+
+        If ```flat=True`` and only one arg is passed can return flat list or just scalar.
 
         If no arguments are passed it will default to a tuple containing all fields
         in order of declaration.
@@ -510,6 +515,8 @@ class QuerySet(AwaitableQuery[MODEL]):
             db=self._db,
             model=self.model,
             q_objects=self._q_objects,
+            single=self._single,
+            raise_does_not_exist=self._raise_does_not_exist,
             flat=flat,
             fields_for_select_list=fields_  # type: ignore
             or [
@@ -533,6 +540,8 @@ class QuerySet(AwaitableQuery[MODEL]):
         """
         Make QuerySet return dicts instead of objects.
 
+        If call after `.get()`, `.get_or_none()` or `.first()` return dict instead of object.
+
         Can pass names of fields to fetch, or as a ``field_name='name_in_dict'`` kwarg.
 
         If no arguments are passed it will default to a dict containing all fields.
@@ -554,7 +563,7 @@ class QuerySet(AwaitableQuery[MODEL]):
             _fields = [
                 field
                 for field in self.model._meta.fields_map.keys()
-                if field in self.model._meta.db_fields
+                if field in self.model._meta.fields_db_projection.keys()
             ] + list(self._annotations.keys())
 
             fields_for_select = {field: field for field in _fields}
@@ -563,6 +572,8 @@ class QuerySet(AwaitableQuery[MODEL]):
             db=self._db,
             model=self.model,
             q_objects=self._q_objects,
+            single=self._single,
+            raise_does_not_exist=self._raise_does_not_exist,
             fields_for_select=fields_for_select,
             distinct=self._distinct,
             limit=self._limit,
@@ -649,6 +660,12 @@ class QuerySet(AwaitableQuery[MODEL]):
         """
         return self._clone()
 
+    def raw(self, sql: str) -> "RawSQLQuery":
+        """
+        Return the QuerySet from raw SQL
+        """
+        return RawSQLQuery(model=self.model, db=self._db, sql=sql)
+
     def first(self) -> QuerySetSingle[Optional[MODEL]]:
         """
         Limit queryset to one object and return one object instead of list.
@@ -668,6 +685,66 @@ class QuerySet(AwaitableQuery[MODEL]):
         queryset._raise_does_not_exist = True
         return queryset  # type: ignore
 
+    async def in_bulk(
+        self, id_list: Iterable[Union[str, int]], field_name: str
+    ) -> Dict[str, MODEL]:
+        """
+        Return a dictionary mapping each of the given IDs to the object with
+        that ID. If `id_list` isn't provided, evaluate the entire QuerySet.
+
+        :param id_list: A list of field values
+        :param field_name: Must be a unique field
+        """
+        objs = await self.filter(**{f"{field_name}__in": id_list})
+        return {getattr(obj, field_name): obj for obj in objs}
+
+    def bulk_create(
+        self,
+        objects: Iterable[MODEL],
+        batch_size: Optional[int] = None,
+        ignore_conflicts: bool = False,
+    ) -> "BulkCreateQuery":
+        """
+        Bulk create objects.
+        """
+        return BulkCreateQuery(  # type:ignore
+            db=self._db,
+            model=self.model,
+            objects=objects,
+            batch_size=batch_size,
+            ignore_conflicts=ignore_conflicts,
+        )
+
+    def bulk_update(
+        self,
+        objects: Iterable[MODEL],
+        fields: Iterable[str],
+        batch_size: Optional[int] = None,
+    ) -> "BulkUpdateQuery":
+        """
+        Update the given fields in each of the given objects in the database.
+
+        :param objects: List of objects to bulk create
+        :param fields: The fields to update
+        :param batch_size: How many objects are created in a single query
+
+        :raises ValueError: If objects have no primary key set
+        """
+        if any(obj.pk is None for obj in objects):
+            raise ValueError("All bulk_update() objects must have a primary key set.")
+        return BulkUpdateQuery(  # type:ignore
+            db=self._db,
+            model=self.model,
+            q_objects=self._q_objects,
+            annotations=self._annotations,
+            custom_filters=self._custom_filters,
+            limit=self._limit,
+            orderings=self._orderings,
+            objects=objects,
+            fields=fields,
+            batch_size=batch_size,
+        )
+
     def get_or_none(self, *args: Q, **kwargs: Any) -> QuerySetSingle[Optional[MODEL]]:
         """
         Fetch exactly one object matching the parameters.
@@ -1299,6 +1376,8 @@ class ValuesListQuery(FieldSelectQuery):
         "annotations",
         "custom_filters",
         "q_objects",
+        "single",
+        "raise_does_not_exist",
         "fields_for_select_list",
         "group_bys",
         "force_indexes",
@@ -1310,6 +1389,8 @@ class ValuesListQuery(FieldSelectQuery):
         model: Type[MODEL],
         db: BaseDBAsyncClient,
         q_objects: List[Q],
+        single: bool,
+        raise_does_not_exist: bool,
         fields_for_select_list: List[str],
         limit: Optional[int],
         offset: Optional[int],
@@ -1334,6 +1415,8 @@ class ValuesListQuery(FieldSelectQuery):
         self.orderings = orderings
         self.custom_filters = custom_filters
         self.q_objects = q_objects
+        self.single = single
+        self.raise_does_not_exist = raise_does_not_exist
         self.fields_for_select_list = fields_for_select_list
         self.flat = flat
         self._db = db
@@ -1371,7 +1454,7 @@ class ValuesListQuery(FieldSelectQuery):
             self.query._use_indexes = []
             self.query = self.query.use_index(*self.use_indexes)
 
-    def __await__(self) -> Generator[Any, None, List[Any]]:
+    def __await__(self) -> Generator[Any, None, Union[List[Any], Tuple]]:
         if self._db is None:
             self._db = self._choose_db()  # type: ignore
         self._make_query()
@@ -1381,7 +1464,7 @@ class ValuesListQuery(FieldSelectQuery):
         for val in await self:
             yield val
 
-    async def _execute(self) -> List[Any]:
+    async def _execute(self) -> Union[List[Any], Tuple]:
         _, result = await self._db.execute_query(str(self.query))
         columns = [
             (key, self.resolve_to_python_value(self.model, name))
@@ -1390,10 +1473,20 @@ class ValuesListQuery(FieldSelectQuery):
         if self.flat:
             func = columns[0][1]
             flatmap = lambda entry: func(entry["0"])  # noqa
-            return list(map(flatmap, result))
+            lst_values = list(map(flatmap, result))
+        else:
+            listmap = lambda entry: tuple(func(entry[column]) for column, func in columns)  # noqa
+            lst_values = list(map(listmap, result))
 
-        listmap = lambda entry: tuple(func(entry[column]) for column, func in columns)  # noqa
-        return list(map(listmap, result))
+        if self.single:
+            if len(lst_values) == 1:
+                return lst_values[0]
+            if not lst_values:
+                if self.raise_does_not_exist:
+                    raise DoesNotExist("Object does not exist")
+                return None  # type: ignore
+            raise MultipleObjectsReturned("Multiple objects returned, expected exactly one")
+        return lst_values
 
 
 class ValuesQuery(FieldSelectQuery):
@@ -1406,6 +1499,8 @@ class ValuesQuery(FieldSelectQuery):
         "annotations",
         "custom_filters",
         "q_objects",
+        "single",
+        "raise_does_not_exist",
         "group_bys",
         "force_indexes",
         "use_indexes",
@@ -1416,6 +1511,8 @@ class ValuesQuery(FieldSelectQuery):
         model: Type[MODEL],
         db: BaseDBAsyncClient,
         q_objects: List[Q],
+        single: bool,
+        raise_does_not_exist: bool,
         fields_for_select: Dict[str, str],
         limit: Optional[int],
         offset: Optional[int],
@@ -1435,6 +1532,8 @@ class ValuesQuery(FieldSelectQuery):
         self.orderings = orderings
         self.custom_filters = custom_filters
         self.q_objects = q_objects
+        self.single = single
+        self.raise_does_not_exist = raise_does_not_exist
         self._db = db
         self.group_bys = group_bys
         self.force_indexes = force_indexes
@@ -1470,7 +1569,7 @@ class ValuesQuery(FieldSelectQuery):
             self.query._use_indexes = []
             self.query = self.query.use_index(*self.use_indexes)
 
-    def __await__(self) -> Generator[Any, None, List[dict]]:
+    def __await__(self) -> Generator[Any, None, Union[List[dict], Dict]]:
         if self._db is None:
             self._db = self._choose_db()  # type: ignore
         self._make_query()
@@ -1480,7 +1579,7 @@ class ValuesQuery(FieldSelectQuery):
         for val in await self:
             yield val
 
-    async def _execute(self) -> List[dict]:
+    async def _execute(self) -> Union[List[dict], Dict]:
         result = await self._db.execute_query_dict(str(self.query))
         columns = [
             val
@@ -1496,4 +1595,166 @@ class ValuesQuery(FieldSelectQuery):
                 for col, func in columns:
                     row[col] = func(row[col])
 
+        if self.single:
+            if len(result) == 1:
+                return result[0]
+            if not result:
+                if self.raise_does_not_exist:
+                    raise DoesNotExist("Object does not exist")
+                return None  # type: ignore
+            raise MultipleObjectsReturned("Multiple objects returned, expected exactly one")
         return result
+
+
+class RawSQLQuery(AwaitableQuery):
+    __slots__ = ("_sql", "_db")
+
+    def __init__(self, model: Type[MODEL], db: BaseDBAsyncClient, sql: str):
+        super().__init__(model)
+        self._sql = sql
+        self._db = db
+
+    def _make_query(self) -> None:
+        self.query = RawSQL(self._sql)
+
+    async def _execute(self) -> Any:
+        instance_list = await self._db.executor_class(
+            model=self.model,
+            db=self._db,
+        ).execute_select(self.query)
+        return instance_list
+
+    def __await__(self) -> Generator[Any, None, List[MODEL]]:
+        if self._db is None:
+            self._db = self._choose_db()  # type: ignore
+        self._make_query()
+        return self._execute().__await__()
+
+
+class BulkUpdateQuery(UpdateQuery):
+    __slots__ = ("objects", "fields", "batch_size", "queries")
+
+    def __init__(
+        self,
+        model: Type[MODEL],
+        db: BaseDBAsyncClient,
+        q_objects: List[Q],
+        annotations: Dict[str, Any],
+        custom_filters: Dict[str, Dict[str, Any]],
+        limit: Optional[int],
+        orderings: List[Tuple[str, str]],
+        objects: Iterable[MODEL],
+        fields: Iterable[str],
+        batch_size: Optional[int] = None,
+    ):
+        super().__init__(model, {}, db, q_objects, annotations, custom_filters, limit, orderings)
+        self.objects = objects
+        self.fields = fields
+        self.batch_size = batch_size
+        self.queries: List[QUERY] = []
+
+    def _make_query(self) -> None:
+        table = self.model._meta.basetable
+        self.query = self._db.query_class.update(table)
+        if self.capabilities.support_update_limit_order_by and self.limit:
+            self.query._limit = self.limit
+            self.resolve_ordering(self.model, table, self.orderings, self.annotations)
+
+        self.resolve_filters(
+            model=self.model,
+            q_objects=self.q_objects,
+            annotations=self.annotations,
+            custom_filters=self.custom_filters,
+        )
+        executor = self._db.executor_class(model=self.model, db=self._db)
+        pk_attr = self.model._meta.pk_attr
+        pk = Field(pk_attr)
+        for objects_item in chunk(self.objects, self.batch_size):
+            query = copy(self.query)
+            for field in self.fields:
+                case = Case()
+                pk_list = []
+                for obj in objects_item:
+                    value = executor.column_map[pk_attr](obj.pk, None)
+                    attr = getattr(obj, field)
+                    case.when(pk == value, attr)
+                    pk_list.append(value)
+                query = query.set(field, case)
+                query = query.where(pk.isin(pk_list))
+            self.queries.append(query)
+
+    async def _execute(self) -> int:
+        count = 0
+        for query in self.queries:
+            count += (await self._db.execute_query(str(query)))[0]
+        return count
+
+    def sql(self, **kwargs) -> str:
+        self.as_query()
+        return ";".join([str(query) for query in self.queries])
+
+
+class BulkCreateQuery(AwaitableQuery):
+    __slots__ = ("objects", "ignore_conflicts", "batch_size", "executor", "_custom_generated_pk")
+
+    def __init__(
+        self,
+        model: Type[MODEL],
+        db: BaseDBAsyncClient,
+        objects: Iterable[MODEL],
+        batch_size: Optional[int] = None,
+        ignore_conflicts: bool = False,
+    ):
+        super().__init__(model)
+        self.objects = list(objects)
+        self.ignore_conflicts = ignore_conflicts
+        self.batch_size = batch_size
+        self._db = db
+        self.executor = self._db.executor_class(model=self.model, db=self._db)
+        self._custom_generated_pk = self.objects[0]._custom_generated_pk
+
+    def _make_query(self) -> None:
+        if not self.ignore_conflicts:
+            if self._custom_generated_pk:
+                self.query = self.executor.insert_query_all
+            else:
+                self.query = self.executor.insert_query
+        else:
+            _, columns = self.executor._prepare_insert_columns()
+            self.query = self.executor._prepare_insert_statement(columns, ignore_conflicts=True)
+            if self.model._meta.generated_db_fields:
+                _, columns_all = self.executor._prepare_insert_columns(include_generated=True)
+                self.query = self.executor._prepare_insert_statement(
+                    columns_all, has_generated=False, ignore_conflicts=True
+                )
+
+    async def _execute(self) -> Any:
+        if self._db is None:
+            self._db = self._choose_db()  # type: ignore
+        self._make_query()
+        for instance_chunk in chunk(self.objects, self.batch_size):
+            values_lists_all = []
+            values_lists = []
+            for instance in instance_chunk:
+                if self._custom_generated_pk:
+                    values_lists_all.append(
+                        [
+                            self.executor.column_map[field_name](
+                                getattr(instance, field_name), instance
+                            )
+                            for field_name in self.executor.regular_columns_all
+                        ]
+                    )
+                else:
+                    values_lists.append(
+                        [
+                            self.executor.column_map[field_name](
+                                getattr(instance, field_name), instance
+                            )
+                            for field_name in self.executor.regular_columns
+                        ]
+                    )
+            if values_lists_all:
+                await self._db.execute_many(self.query, values_lists_all)
+            if values_lists:
+                await self._db.execute_many(self.query, values_lists)
